@@ -1,42 +1,29 @@
 //! Process data collection for Linux.
 
-use std::collections::hash_map::Entry;
-
-use crate::utils::error::{self, BottomError};
-use crate::Pid;
-
-use super::ProcessHarvest;
-
-use sysinfo::ProcessStatus;
-
-use procfs::process::{Process, Stat};
+use std::fs::File;
+use std::io::{BufRead, BufReader};
 
 use fxhash::{FxHashMap, FxHashSet};
+use procfs::process::{Process, Stat};
+use sysinfo::ProcessStatus;
+
+use super::{ProcessHarvest, UserTable};
+use crate::components::tui_widget::time_chart::Point;
+use crate::utils::error::{self, BottomError};
+use crate::Pid;
 
 /// Maximum character length of a /proc/<PID>/stat process name.
 /// If it's equal or greater, then we instead refer to the command for the name.
 const MAX_STAT_NAME_LEN: usize = 15;
 
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Default)]
 pub struct PrevProcDetails {
-    pub total_read_bytes: u64,
-    pub total_write_bytes: u64,
-    pub cpu_time: u64,
-    pub process: Process,
+    total_read_bytes: u64,
+    total_write_bytes: u64,
+    cpu_time: u64,
 }
 
-impl PrevProcDetails {
-    fn new(pid: Pid) -> error::Result<Self> {
-        Ok(Self {
-            total_read_bytes: 0,
-            total_write_bytes: 0,
-            cpu_time: 0,
-            process: Process::new(pid)?,
-        })
-    }
-}
-
-fn calculate_idle_values(line: String) -> (f64, f64) {
+fn calculate_idle_values(line: &str) -> Point {
     /// Converts a `Option<&str>` value to an f64. If it fails to parse or is `None`, then it will return `0_f64`.
     fn str_to_f64(val: Option<&str>) -> f64 {
         val.and_then(|v| v.parse::<f64>().ok()).unwrap_or(0_f64)
@@ -61,70 +48,78 @@ fn calculate_idle_values(line: String) -> (f64, f64) {
     (idle, non_idle)
 }
 
-fn cpu_usage_calculation(
-    prev_idle: &mut f64, prev_non_idle: &mut f64,
-) -> error::Result<(f64, f64)> {
-    use std::io::prelude::*;
-    use std::io::BufReader;
+struct CpuUsage {
+    /// Difference between the total delta and the idle delta.
+    cpu_usage: f64,
 
-    // From SO answer: https://stackoverflow.com/a/23376195
-    let mut reader = BufReader::new(std::fs::File::open("/proc/stat")?);
-    let mut first_line = String::new();
-    reader.read_line(&mut first_line)?;
+    /// Overall CPU usage as a fraction.
+    cpu_fraction: f64,
+}
 
-    let (idle, non_idle) = calculate_idle_values(first_line);
+fn cpu_usage_calculation(prev_idle: &mut f64, prev_non_idle: &mut f64) -> error::Result<CpuUsage> {
+    let (idle, non_idle) = {
+        // From SO answer: https://stackoverflow.com/a/23376195
+        let mut reader = BufReader::new(File::open("/proc/stat")?);
+        let mut first_line = String::new();
+        reader.read_line(&mut first_line)?;
+
+        calculate_idle_values(&first_line)
+    };
 
     let total = idle + non_idle;
     let prev_total = *prev_idle + *prev_non_idle;
 
-    let total_delta: f64 = total - prev_total;
-    let idle_delta: f64 = idle - *prev_idle;
+    let total_delta = total - prev_total;
+    let idle_delta = idle - *prev_idle;
 
     *prev_idle = idle;
     *prev_non_idle = non_idle;
 
-    let result = if total_delta - idle_delta != 0_f64 {
+    // TODO: Should these return errors instead?
+    let cpu_usage = if total_delta - idle_delta != 0.0 {
         total_delta - idle_delta
     } else {
-        1_f64
+        1.0
     };
 
-    let cpu_percentage = if total_delta != 0_f64 {
-        result / total_delta
+    let cpu_fraction = if total_delta != 0.0 {
+        cpu_usage / total_delta
     } else {
-        0_f64
+        0.0
     };
 
-    Ok((result, cpu_percentage))
+    Ok(CpuUsage {
+        cpu_usage,
+        cpu_fraction,
+    })
 }
 
-/// Returns the usage and a new set of process times. Note: cpu_fraction should be represented WITHOUT the x100 factor!
+/// Returns the usage and a new set of process times.
+///
+/// NB: cpu_fraction should be represented WITHOUT the x100 factor!
 fn get_linux_cpu_usage(
     stat: &Stat, cpu_usage: f64, cpu_fraction: f64, prev_proc_times: u64,
     use_current_cpu_total: bool,
 ) -> (f64, u64) {
     // Based heavily on https://stackoverflow.com/a/23376195 and https://stackoverflow.com/a/1424556
     let new_proc_times = stat.utime + stat.stime;
-    let diff = (new_proc_times - prev_proc_times) as f64; // I HATE that it's done like this but there isn't a try_from for u64 -> f64... we can accept a bit of loss in the worst case though
+    let diff = (new_proc_times - prev_proc_times) as f64; // No try_from for u64 -> f64... oh well.
 
     if cpu_usage == 0.0 {
         (0.0, new_proc_times)
     } else if use_current_cpu_total {
-        (diff / cpu_usage * 100_f64, new_proc_times)
+        ((diff / cpu_usage) * 100.0, new_proc_times)
     } else {
-        (diff / cpu_usage * 100_f64 * cpu_fraction, new_proc_times)
+        ((diff / cpu_usage) * 100.0 * cpu_fraction, new_proc_times)
     }
 }
 
-#[allow(clippy::too_many_arguments)]
 fn read_proc(
-    prev_proc: &PrevProcDetails, stat: &Stat, cpu_usage: f64, cpu_fraction: f64,
+    prev_proc: &PrevProcDetails, process: &Process, cpu_usage: f64, cpu_fraction: f64,
     use_current_cpu_total: bool, time_difference_in_secs: u64, mem_total_kb: u64,
+    user_table: &mut UserTable,
 ) -> error::Result<(ProcessHarvest, u64)> {
-    use std::convert::TryFrom;
-
-    let process = &prev_proc.process;
-
+    let stat = process.stat()?;
     let (command, name) = {
         let truncated_name = stat.comm.as_str();
         if let Ok(cmdline) = process.cmdline() {
@@ -156,38 +151,39 @@ fn read_proc(
     };
 
     let process_state_char = stat.state;
-    let process_state = ProcessStatus::from(process_state_char).to_string();
+    let process_state = (
+        ProcessStatus::from(process_state_char).to_string(),
+        process_state_char,
+    );
     let (cpu_usage_percent, new_process_times) = get_linux_cpu_usage(
-        stat,
+        &stat,
         cpu_usage,
         cpu_fraction,
         prev_proc.cpu_time,
         use_current_cpu_total,
     );
     let parent_pid = Some(stat.ppid);
-    let mem_usage_bytes = u64::try_from(stat.rss_bytes()?).unwrap_or(0);
+    let mem_usage_bytes = stat.rss_bytes()?;
     let mem_usage_kb = mem_usage_bytes / 1024;
     let mem_usage_percent = mem_usage_kb as f64 / mem_total_kb as f64 * 100.0;
 
     // This can fail if permission is denied!
-
     let (total_read_bytes, total_write_bytes, read_bytes_per_sec, write_bytes_per_sec) =
         if let Ok(io) = process.io() {
             let total_read_bytes = io.read_bytes;
             let total_write_bytes = io.write_bytes;
+            let prev_total_read_bytes = prev_proc.total_read_bytes;
+            let prev_total_write_bytes = prev_proc.total_write_bytes;
 
-            let read_bytes_per_sec = if time_difference_in_secs == 0 {
-                0
-            } else {
-                total_read_bytes.saturating_sub(prev_proc.total_read_bytes)
-                    / time_difference_in_secs
-            };
-            let write_bytes_per_sec = if time_difference_in_secs == 0 {
-                0
-            } else {
-                total_write_bytes.saturating_sub(prev_proc.total_write_bytes)
-                    / time_difference_in_secs
-            };
+            let read_bytes_per_sec = total_read_bytes
+                .saturating_sub(prev_total_read_bytes)
+                .checked_div(time_difference_in_secs)
+                .unwrap_or(0);
+
+            let write_bytes_per_sec = total_write_bytes
+                .saturating_sub(prev_total_write_bytes)
+                .checked_div(time_difference_in_secs)
+                .unwrap_or(0);
 
             (
                 total_read_bytes,
@@ -199,7 +195,7 @@ fn read_proc(
             (0, 0, 0, 0)
         };
 
-    let uid = Some(process.owner);
+    let uid = process.uid()?;
 
     Ok((
         ProcessHarvest {
@@ -215,69 +211,75 @@ fn read_proc(
             total_read_bytes,
             total_write_bytes,
             process_state,
-            process_state_char,
-            uid,
+            uid: Some(uid),
+            user: user_table
+                .get_uid_to_username_mapping(uid)
+                .map(Into::into)
+                .unwrap_or_else(|_| "N/A".into()),
         },
         new_process_times,
     ))
 }
 
+/// How to calculate CPU usage.
+pub enum CpuUsageStrategy {
+    /// Normalized means the displayed usage percentage is divided over the number of CPU cores.
+    ///
+    /// For example, if the "overall" usage over the entire system is 105%, and there are 5 cores, then
+    /// the displayed percentage is 21%.
+    Normalized,
+
+    /// Non-normalized means that the overall usage over the entire system is shown, without dividing
+    /// over the number of cores.
+    NonNormalized(f64),
+}
+
 pub fn get_process_data(
     prev_idle: &mut f64, prev_non_idle: &mut f64,
     pid_mapping: &mut FxHashMap<Pid, PrevProcDetails>, use_current_cpu_total: bool,
-    time_difference_in_secs: u64, mem_total_kb: u64,
+    normalization: CpuUsageStrategy, time_difference_in_secs: u64, mem_total_kb: u64,
+    user_table: &mut UserTable,
 ) -> crate::utils::error::Result<Vec<ProcessHarvest>> {
     // TODO: [PROC THREADS] Add threads
 
-    if let Ok((cpu_usage, cpu_fraction)) = cpu_usage_calculation(prev_idle, prev_non_idle) {
+    if let Ok(CpuUsage {
+        mut cpu_usage,
+        cpu_fraction,
+    }) = cpu_usage_calculation(prev_idle, prev_non_idle)
+    {
+        if let CpuUsageStrategy::NonNormalized(num_cores) = normalization {
+            // Note we *divide* here because the later calculation divides `cpu_usage` - in effect,
+            // multiplying over the number of cores.
+            cpu_usage /= num_cores;
+        }
+
         let mut pids_to_clear: FxHashSet<Pid> = pid_mapping.keys().cloned().collect();
 
         let process_vector: Vec<ProcessHarvest> = std::fs::read_dir("/proc")?
             .filter_map(|dir| {
                 if let Ok(dir) = dir {
                     if let Ok(pid) = dir.file_name().to_string_lossy().trim().parse::<Pid>() {
-                        let mut fresh = false;
-                        if let Entry::Vacant(entry) = pid_mapping.entry(pid) {
-                            if let Ok(ppd) = PrevProcDetails::new(pid) {
-                                entry.insert(ppd);
-                                fresh = true;
-                            } else {
-                                // Bail early.
-                                return None;
-                            }
+                        let Ok(process) = Process::new(pid) else {
+                            return None;
                         };
+                        let prev_proc_details = pid_mapping.entry(pid).or_default();
 
-                        if let Some(prev_proc_details) = pid_mapping.get_mut(&pid) {
-                            let stat;
-                            let stat_live;
-                            if fresh {
-                                stat = &prev_proc_details.process.stat;
-                            } else if let Ok(s) = prev_proc_details.process.stat() {
-                                stat_live = s;
-                                stat = &stat_live;
-                            } else {
-                                // Bail early.
-                                return None;
-                            }
+                        if let Ok((process_harvest, new_process_times)) = read_proc(
+                            prev_proc_details,
+                            &process,
+                            cpu_usage,
+                            cpu_fraction,
+                            use_current_cpu_total,
+                            time_difference_in_secs,
+                            mem_total_kb,
+                            user_table,
+                        ) {
+                            prev_proc_details.cpu_time = new_process_times;
+                            prev_proc_details.total_read_bytes = process_harvest.total_read_bytes;
+                            prev_proc_details.total_write_bytes = process_harvest.total_write_bytes;
 
-                            if let Ok((process_harvest, new_process_times)) = read_proc(
-                                prev_proc_details,
-                                stat,
-                                cpu_usage,
-                                cpu_fraction,
-                                use_current_cpu_total,
-                                time_difference_in_secs,
-                                mem_total_kb,
-                            ) {
-                                prev_proc_details.cpu_time = new_process_times;
-                                prev_proc_details.total_read_bytes =
-                                    process_harvest.total_read_bytes;
-                                prev_proc_details.total_write_bytes =
-                                    process_harvest.total_write_bytes;
-
-                                pids_to_clear.remove(&pid);
-                                return Some(process_harvest);
-                            }
+                            pids_to_clear.remove(&pid);
+                            return Some(process_harvest);
                         }
                     }
                 }
@@ -306,37 +308,37 @@ mod tests {
     fn test_proc_cpu_parse() {
         assert_eq!(
             (100_f64, 200_f64),
-            calculate_idle_values("100 0 100 100".to_string()),
+            calculate_idle_values("100 0 100 100"),
             "Failed to properly calculate idle/non-idle for /proc/stat CPU with 4 values"
         );
         assert_eq!(
             (120_f64, 200_f64),
-            calculate_idle_values("100 0 100 100 20".to_string()),
+            calculate_idle_values("100 0 100 100 20"),
             "Failed to properly calculate idle/non-idle for /proc/stat CPU with 5 values"
         );
         assert_eq!(
             (120_f64, 230_f64),
-            calculate_idle_values("100 0 100 100 20 30".to_string()),
+            calculate_idle_values("100 0 100 100 20 30"),
             "Failed to properly calculate idle/non-idle for /proc/stat CPU with 6 values"
         );
         assert_eq!(
             (120_f64, 270_f64),
-            calculate_idle_values("100 0 100 100 20 30 40".to_string()),
+            calculate_idle_values("100 0 100 100 20 30 40"),
             "Failed to properly calculate idle/non-idle for /proc/stat CPU with 7 values"
         );
         assert_eq!(
             (120_f64, 320_f64),
-            calculate_idle_values("100 0 100 100 20 30 40 50".to_string()),
+            calculate_idle_values("100 0 100 100 20 30 40 50"),
             "Failed to properly calculate idle/non-idle for /proc/stat CPU with 8 values"
         );
         assert_eq!(
             (120_f64, 320_f64),
-            calculate_idle_values("100 0 100 100 20 30 40 50 100".to_string()),
+            calculate_idle_values("100 0 100 100 20 30 40 50 100"),
             "Failed to properly calculate idle/non-idle for /proc/stat CPU with 9 values"
         );
         assert_eq!(
             (120_f64, 320_f64),
-            calculate_idle_values("100 0 100 100 20 30 40 50 100 200".to_string()),
+            calculate_idle_values("100 0 100 100 20 30 40 50 100 200"),
             "Failed to properly calculate idle/non-idle for /proc/stat CPU with 10 values"
         );
     }
