@@ -1,10 +1,12 @@
 //! Unix-specific parts of process collection.
 
+use fxhash::FxHashMap;
+
 use crate::utils::error;
 
 #[derive(Debug, Default)]
 pub struct UserTable {
-    pub uid_user_mapping: std::collections::HashMap<libc::uid_t, String>,
+    pub uid_user_mapping: FxHashMap<libc::uid_t, String>,
 }
 
 impl UserTable {
