@@ -1,15 +1,17 @@
 //! Process data collection for Windows.  Uses sysinfo.
 
+use sysinfo::{CpuExt, PidExt, ProcessExt, System, SystemExt};
+
 use super::ProcessHarvest;
-use sysinfo::{PidExt, ProcessExt, ProcessorExt, System, SystemExt};
 
 pub fn get_process_data(
-    sys: &System, use_current_cpu_total: bool, mem_total_kb: u64,
+    sys: &System, use_current_cpu_total: bool, unnormalized_cpu: bool, mem_total_kb: u64,
 ) -> crate::utils::error::Result<Vec<ProcessHarvest>> {
     let mut process_vector: Vec<ProcessHarvest> = Vec::new();
     let process_hashmap = sys.processes();
-    let cpu_usage = sys.global_processor_info().cpu_usage() as f64 / 100.0;
-    let num_processors = sys.processors().len() as f64;
+    let cpu_usage = sys.global_cpu_info().cpu_usage() as f64 / 100.0;
+    let num_processors = sys.cpus().len();
+
     for process_val in process_hashmap.values() {
         let name = if process_val.name().is_empty() {
             let process_cmd = process_val.cmd();
@@ -41,11 +43,11 @@ pub fn get_process_data(
         };
 
         let pcu = {
-            let p = process_val.cpu_usage() as f64 / num_processors;
-            if p.is_nan() {
-                process_val.cpu_usage() as f64
+            let usage = process_val.cpu_usage() as f64;
+            if unnormalized_cpu || num_processors == 0 {
+                usage
             } else {
-                p
+                usage / (num_processors as f64)
             }
         };
         let process_cpu_usage = if use_current_cpu_total && cpu_usage > 0.0 {
@@ -55,6 +57,7 @@ pub fn get_process_data(
         };
 
         let disk_usage = process_val.disk_usage();
+        let process_state = (process_val.status().to_string(), 'R');
         process_vector.push(ProcessHarvest {
             pid: process_val.pid().as_u32() as _,
             parent_pid: process_val.parent().map(|p| p.as_u32() as _),
@@ -65,14 +68,13 @@ pub fn get_process_data(
             } else {
                 0.0
             },
-            mem_usage_bytes: process_val.memory() * 1024,
+            mem_usage_bytes: process_val.memory(),
             cpu_usage_percent: process_cpu_usage,
             read_bytes_per_sec: disk_usage.read_bytes,
             write_bytes_per_sec: disk_usage.written_bytes,
             total_read_bytes: disk_usage.total_read_bytes,
             total_write_bytes: disk_usage.total_written_bytes,
-            process_state: process_val.status().to_string(),
-            process_state_char: 'R',
+            process_state,
         });
     }
 
