@@ -13,13 +13,25 @@ use crate::units::data_units::DataUnit;
 use crate::utils::gen_util::*;
 use crate::widgets::{DiskWidgetData, TempWidgetData};
 
+#[derive(Debug)]
+pub enum BatteryDuration {
+    ToEmpty(i64),
+    ToFull(i64),
+    Unknown,
+}
+
+impl Default for BatteryDuration {
+    fn default() -> Self {
+        BatteryDuration::Unknown
+    }
+}
+
 #[derive(Default, Debug)]
 pub struct ConvertedBatteryData {
     pub battery_name: String,
     pub charge_percentage: f64,
     pub watt_consumption: String,
-    pub duration_until_full: Option<String>,
-    pub duration_until_empty: Option<String>,
+    pub battery_duration: BatteryDuration,
     pub health: String,
 }
 
@@ -63,7 +75,7 @@ pub struct ConvertedData {
     pub mem_labels: Option<(String, String)>,
     pub swap_labels: Option<(String, String)>,
 
-    pub mem_data: Vec<Point>, // TODO: Switch this and all data points over to a better data structure...
+    pub mem_data: Vec<Point>, /* TODO: Switch this and all data points over to a better data structure... */
     pub swap_data: Vec<Point>,
 
     #[cfg(feature = "zfs")]
@@ -90,12 +102,18 @@ impl ConvertedData {
             .iter()
             .zip(&data.io_labels)
             .for_each(|(disk, (io_read, io_write))| {
+                let summed_total_bytes = match (disk.used_space, disk.free_space) {
+                    (Some(used), Some(free)) => Some(used + free),
+                    _ => None,
+                };
+
                 self.disk_data.push(DiskWidgetData {
                     name: KString::from_ref(&disk.name),
                     mount_point: KString::from_ref(&disk.mount_point),
                     free_bytes: disk.free_space,
                     used_bytes: disk.used_space,
                     total_bytes: disk.total_space,
+                    summed_total_bytes,
                     io_read: io_read.into(),
                     io_write: io_write.into(),
                 });
@@ -374,7 +392,7 @@ pub fn convert_network_data_points(
     let (rx_converted_result, total_rx_converted_result): ((f64, String), (f64, &'static str)) =
         if network_use_binary_prefix {
             (
-                get_binary_prefix(rx_data, unit), // If this isn't obvious why there's two functions, one you can configure the unit, the other is always bytes
+                get_binary_prefix(rx_data, unit), /* If this isn't obvious why there's two functions, one you can configure the unit, the other is always bytes */
                 get_binary_bytes(total_rx_data),
             )
         } else {
@@ -521,37 +539,12 @@ pub fn convert_battery_harvest(current_data: &DataCollection) -> Vec<ConvertedBa
             battery_name: format!("Battery {}", itx),
             charge_percentage: battery_harvest.charge_percent,
             watt_consumption: format!("{:.2}W", battery_harvest.power_consumption_rate_watts),
-            duration_until_empty: if let Some(secs_till_empty) = battery_harvest.secs_until_empty {
-                let time = time::Duration::seconds(secs_till_empty);
-                let num_minutes = time.whole_minutes() - time.whole_hours() * 60;
-                let num_seconds = time.whole_seconds() - time.whole_minutes() * 60;
-                Some(format!(
-                    "{} hour{}, {} minute{}, {} second{}",
-                    time.whole_hours(),
-                    if time.whole_hours() == 1 { "" } else { "s" },
-                    num_minutes,
-                    if num_minutes == 1 { "" } else { "s" },
-                    num_seconds,
-                    if num_seconds == 1 { "" } else { "s" },
-                ))
+            battery_duration: if let Some(secs) = battery_harvest.secs_until_empty {
+                BatteryDuration::ToEmpty(secs)
+            } else if let Some(secs) = battery_harvest.secs_until_full {
+                BatteryDuration::ToFull(secs)
             } else {
-                None
-            },
-            duration_until_full: if let Some(secs_till_full) = battery_harvest.secs_until_full {
-                let time = time::Duration::seconds(secs_till_full);
-                let num_minutes = time.whole_minutes() - time.whole_hours() * 60;
-                let num_seconds = time.whole_seconds() - time.whole_minutes() * 60;
-                Some(format!(
-                    "{} hour{}, {} minute{}, {} second{}",
-                    time.whole_hours(),
-                    if time.whole_hours() == 1 { "" } else { "s" },
-                    num_minutes,
-                    if num_minutes == 1 { "" } else { "s" },
-                    num_seconds,
-                    if num_seconds == 1 { "" } else { "s" },
-                ))
-            } else {
-                None
+                BatteryDuration::Unknown
             },
             health: format!("{:.2}%", battery_harvest.health_percent),
         })
