@@ -1,25 +1,29 @@
-use regex::Regex;
-use serde::{Deserialize, Serialize};
 use std::{
     borrow::Cow,
     collections::{HashMap, HashSet},
-    path::PathBuf,
+    convert::TryInto,
     str::FromStr,
-    time::Instant,
+    time::{Duration, Instant},
 };
 
+use clap::ArgMatches;
+use layout_options::*;
+use regex::Regex;
+use serde::{Deserialize, Serialize};
+use typed_builder::*;
+
 use crate::{
     app::{layout_manager::*, *},
-    canvas::ColourScheme,
+    canvas::{canvas_colours::CanvasColours, ColourScheme},
     constants::*,
     units::data_units::DataUnit,
     utils::error::{self, BottomError},
+    widgets::{
+        BatteryWidgetState, CpuWidgetState, DiskTableWidget, MemWidgetState, NetWidgetState,
+        ProcWidget, ProcWidgetMode, TempWidgetState,
+    },
 };
 
-use typed_builder::*;
-
-use layout_options::*;
-
 pub mod layout_options;
 
 use anyhow::{Context, Result};
@@ -49,123 +53,44 @@ impl Config {
 
 #[derive(Clone, Debug, Default, Deserialize, Serialize, TypedBuilder)]
 pub struct ConfigFlags {
-    #[builder(default, setter(strip_option))]
     pub hide_avg_cpu: Option<bool>,
-
-    #[builder(default, setter(strip_option))]
     pub dot_marker: Option<bool>,
-
-    #[builder(default, setter(strip_option))]
     pub temperature_type: Option<String>,
-
-    #[builder(default, setter(strip_option))]
     pub rate: Option<u64>,
-
-    #[builder(default, setter(strip_option))]
     pub left_legend: Option<bool>,
-
-    #[builder(default, setter(strip_option))]
     pub current_usage: Option<bool>,
-
-    #[builder(default, setter(strip_option))]
+    pub unnormalized_cpu: Option<bool>,
     pub group_processes: Option<bool>,
-
-    #[builder(default, setter(strip_option))]
     pub case_sensitive: Option<bool>,
-
-    #[builder(default, setter(strip_option))]
     pub whole_word: Option<bool>,
-
-    #[builder(default, setter(strip_option))]
     pub regex: Option<bool>,
-
-    #[builder(default, setter(strip_option))]
     pub basic: Option<bool>,
-
-    #[builder(default, setter(strip_option))]
     pub default_time_value: Option<u64>,
-
-    #[builder(default, setter(strip_option))]
     pub time_delta: Option<u64>,
-
-    #[builder(default, setter(strip_option))]
     pub autohide_time: Option<bool>,
-
-    #[builder(default, setter(strip_option))]
     pub hide_time: Option<bool>,
-
-    #[builder(default, setter(strip_option))]
     pub default_widget_type: Option<String>,
-
-    #[builder(default, setter(strip_option))]
     pub default_widget_count: Option<u64>,
-
-    #[builder(default, setter(strip_option))]
+    pub expanded_on_startup: Option<bool>,
     pub use_old_network_legend: Option<bool>,
-
-    #[builder(default, setter(strip_option))]
     pub hide_table_gap: Option<bool>,
-
-    #[builder(default, setter(strip_option))]
     pub battery: Option<bool>,
-
-    #[builder(default, setter(strip_option))]
     pub disable_click: Option<bool>,
-
-    #[builder(default, setter(strip_option))]
     pub no_write: Option<bool>,
-
     // For built-in colour palettes.
-    #[builder(default, setter(strip_option))]
     pub color: Option<String>,
-
-    // This is a huge hack to enable hashmap functionality WITHOUT being able to serializing the field.
-    // Basically, keep a hashmap in the struct, and convert to a vector every time.
-    #[builder(default, setter(strip_option))]
-    #[serde(skip)]
-    pub search_case_enabled_widgets_map: Option<HashMap<u64, bool>>,
-
-    #[builder(default, setter(strip_option))]
-    pub search_case_enabled_widgets: Option<Vec<WidgetIdEnabled>>,
-
-    #[builder(default, setter(strip_option))]
-    #[serde(skip)]
-    pub search_whole_word_enabled_widgets_map: Option<HashMap<u64, bool>>,
-
-    #[builder(default, setter(strip_option))]
-    pub search_whole_word_enabled_widgets: Option<Vec<WidgetIdEnabled>>,
-
-    #[builder(default, setter(strip_option))]
-    #[serde(skip)]
-    pub search_regex_enabled_widgets_map: Option<HashMap<u64, bool>>,
-
-    #[builder(default, setter(strip_option))]
-    pub search_regex_enabled_widgets: Option<Vec<WidgetIdEnabled>>,
-
-    // End hack
-    #[builder(default, setter(strip_option))]
     pub mem_as_value: Option<bool>,
-
-    #[builder(default, setter(strip_option))]
     pub tree: Option<bool>,
-
-    #[builder(default, setter(strip_option))]
     show_table_scroll_position: Option<bool>,
-
-    #[builder(default, setter(strip_option))]
     pub process_command: Option<bool>,
-
-    #[builder(default, setter(strip_option))]
     pub disable_advanced_kill: Option<bool>,
-
-    #[builder(default, setter(strip_option))]
     pub network_use_bytes: Option<bool>,
-
-    #[builder(default, setter(strip_option))]
     pub network_use_log: Option<bool>,
-
-    #[builder(default, setter(strip_option))]
     pub network_use_binary_prefix: Option<bool>,
+    pub enable_gpu_memory: Option<bool>,
+    #[serde(with = "humantime_serde")]
+    #[serde(default)]
+    pub retention: Option<Duration>,
 }
 
 #[derive(Clone, Default, Debug, Deserialize, Serialize)]
@@ -188,27 +113,29 @@ impl WidgetIdEnabled {
 
 #[derive(Clone, Debug, Default, Deserialize, Serialize)]
 pub struct ConfigColours {
-    pub table_header_color: Option<String>,
-    pub all_cpu_color: Option<String>,
-    pub avg_cpu_color: Option<String>,
-    pub cpu_core_colors: Option<Vec<String>>,
-    pub ram_color: Option<String>,
-    pub swap_color: Option<String>,
-    pub rx_color: Option<String>,
-    pub tx_color: Option<String>,
-    pub rx_total_color: Option<String>, // These only affect basic mode.
-    pub tx_total_color: Option<String>, // These only affect basic mode.
-    pub border_color: Option<String>,
-    pub highlighted_border_color: Option<String>,
-    pub disabled_text_color: Option<String>,
-    pub text_color: Option<String>,
-    pub selected_text_color: Option<String>,
-    pub selected_bg_color: Option<String>,
-    pub widget_title_color: Option<String>,
-    pub graph_color: Option<String>,
-    pub high_battery_color: Option<String>,
-    pub medium_battery_color: Option<String>,
-    pub low_battery_color: Option<String>,
+    pub table_header_color: Option<Cow<'static, str>>,
+    pub all_cpu_color: Option<Cow<'static, str>>,
+    pub avg_cpu_color: Option<Cow<'static, str>>,
+    pub cpu_core_colors: Option<Vec<Cow<'static, str>>>,
+    pub ram_color: Option<Cow<'static, str>>,
+    pub swap_color: Option<Cow<'static, str>>,
+    pub arc_color: Option<Cow<'static, str>>,
+    pub gpu_core_colors: Option<Vec<Cow<'static, str>>>,
+    pub rx_color: Option<Cow<'static, str>>,
+    pub tx_color: Option<Cow<'static, str>>,
+    pub rx_total_color: Option<Cow<'static, str>>, // These only affect basic mode.
+    pub tx_total_color: Option<Cow<'static, str>>, // These only affect basic mode.
+    pub border_color: Option<Cow<'static, str>>,
+    pub highlighted_border_color: Option<Cow<'static, str>>,
+    pub disabled_text_color: Option<Cow<'static, str>>,
+    pub text_color: Option<Cow<'static, str>>,
+    pub selected_text_color: Option<Cow<'static, str>>,
+    pub selected_bg_color: Option<Cow<'static, str>>,
+    pub widget_title_color: Option<Cow<'static, str>>,
+    pub graph_color: Option<Cow<'static, str>>,
+    pub high_battery_color: Option<Cow<'static, str>>,
+    pub medium_battery_color: Option<Cow<'static, str>>,
+    pub low_battery_color: Option<Cow<'static, str>>,
 }
 
 impl ConfigColours {
@@ -244,13 +171,16 @@ pub struct IgnoreList {
 }
 
 pub fn build_app(
-    matches: &clap::ArgMatches<'static>, config: &mut Config, widget_layout: &BottomLayout,
+    matches: &ArgMatches, config: &mut Config, widget_layout: &BottomLayout,
     default_widget_id: u64, default_widget_type_option: &Option<BottomWidgetType>,
-    config_path: Option<PathBuf>,
+    colours: &CanvasColours,
 ) -> Result<App> {
     use BottomWidgetType::*;
+
+    let retention_ms =
+        get_retention_ms(matches, config).context("Update `retention` in your config file.")?;
     let autohide_time = get_autohide_time(matches, config);
-    let default_time_value = get_default_time_value(matches, config)
+    let default_time_value = get_default_time_value(matches, config, retention_ms)
         .context("Update 'default_time_value' in your config file.")?;
     let use_basic_mode = get_use_basic_mode(matches, config);
 
@@ -264,9 +194,9 @@ pub fn build_app(
     let mut cpu_state_map: HashMap<u64, CpuWidgetState> = HashMap::new();
     let mut mem_state_map: HashMap<u64, MemWidgetState> = HashMap::new();
     let mut net_state_map: HashMap<u64, NetWidgetState> = HashMap::new();
-    let mut proc_state_map: HashMap<u64, ProcWidgetState> = HashMap::new();
+    let mut proc_state_map: HashMap<u64, ProcWidget> = HashMap::new();
     let mut temp_state_map: HashMap<u64, TempWidgetState> = HashMap::new();
-    let mut disk_state_map: HashMap<u64, DiskWidgetState> = HashMap::new();
+    let mut disk_state_map: HashMap<u64, DiskTableWidget> = HashMap::new();
     let mut battery_state_map: HashMap<u64, BatteryWidgetState> = HashMap::new();
 
     let autohide_timer = if autohide_time {
@@ -289,6 +219,34 @@ pub fn build_app(
     let network_scale_type = get_network_scale_type(matches, config);
     let network_use_binary_prefix = get_network_use_binary_prefix(matches, config);
 
+    let app_config_fields = AppConfigFields {
+        update_rate_in_milliseconds: get_update_rate_in_milliseconds(matches, config)
+            .context("Update 'rate' in your config file.")?,
+        temperature_type: get_temperature(matches, config)
+            .context("Update 'temperature_type' in your config file.")?,
+        show_average_cpu: get_show_average_cpu(matches, config),
+        use_dot: get_use_dot(matches, config),
+        left_legend: get_use_left_legend(matches, config),
+        use_current_cpu_total: get_use_current_cpu_total(matches, config),
+        unnormalized_cpu: get_unnormalized_cpu(matches, config),
+        use_basic_mode,
+        default_time_value,
+        time_interval: get_time_interval(matches, config, retention_ms)
+            .context("Update 'time_delta' in your config file.")?,
+        hide_time: get_hide_time(matches, config),
+        autohide_time,
+        use_old_network_legend: get_use_old_network_legend(matches, config),
+        table_gap: u16::from(!get_hide_table_gap(matches, config)),
+        disable_click: get_disable_click(matches, config),
+        enable_gpu_memory: get_enable_gpu_memory(matches, config),
+        show_table_scroll_position: get_show_table_scroll_position(matches, config),
+        is_advanced_kill,
+        network_scale_type,
+        network_unit_type,
+        network_use_binary_prefix,
+        retention_ms,
+    };
+
     for row in &widget_layout.rows {
         for col in &row.children {
             for col_row in &col.children {
@@ -331,7 +289,12 @@ pub fn build_app(
                         Cpu => {
                             cpu_state_map.insert(
                                 widget.widget_id,
-                                CpuWidgetState::init(default_time_value, autohide_timer),
+                                CpuWidgetState::new(
+                                    &app_config_fields,
+                                    default_time_value,
+                                    autohide_timer,
+                                    colours,
+                                ),
                             );
                         }
                         Mem => {
@@ -343,33 +306,45 @@ pub fn build_app(
                         Net => {
                             net_state_map.insert(
                                 widget.widget_id,
-                                NetWidgetState::init(
-                                    default_time_value,
-                                    autohide_timer,
-                                    // network_unit_type.clone(),
-                                    // network_scale_type.clone(),
-                                ),
+                                NetWidgetState::init(default_time_value, autohide_timer),
                             );
                         }
                         Proc => {
+                            let mode = if is_grouped {
+                                ProcWidgetMode::Grouped
+                            } else if is_default_tree {
+                                ProcWidgetMode::Tree {
+                                    collapsed_pids: Default::default(),
+                                }
+                            } else {
+                                ProcWidgetMode::Normal
+                            };
+
                             proc_state_map.insert(
                                 widget.widget_id,
-                                ProcWidgetState::init(
+                                ProcWidget::new(
+                                    &app_config_fields,
+                                    mode,
                                     is_case_sensitive,
                                     is_match_whole_word,
                                     is_use_regex,
-                                    is_grouped,
                                     show_memory_as_values,
-                                    is_default_tree,
                                     is_default_command,
+                                    colours,
                                 ),
                             );
                         }
                         Disk => {
-                            disk_state_map.insert(widget.widget_id, DiskWidgetState::init());
+                            disk_state_map.insert(
+                                widget.widget_id,
+                                DiskTableWidget::new(&app_config_fields, colours),
+                            );
                         }
                         Temp => {
-                            temp_state_map.insert(widget.widget_id, TempWidgetState::init());
+                            temp_state_map.insert(
+                                widget.widget_id,
+                                TempWidgetState::new(&app_config_fields, colours),
+                            );
                         }
                         Battery => {
                             battery_state_map
@@ -407,40 +382,11 @@ pub fn build_app(
         None
     };
 
-    let app_config_fields = AppConfigFields {
-        update_rate_in_milliseconds: get_update_rate_in_milliseconds(matches, config)
-            .context("Update 'rate' in your config file.")?,
-        temperature_type: get_temperature(matches, config)
-            .context("Update 'temperature_type' in your config file.")?,
-        show_average_cpu: get_show_average_cpu(matches, config),
-        use_dot: get_use_dot(matches, config),
-        left_legend: get_use_left_legend(matches, config),
-        use_current_cpu_total: get_use_current_cpu_total(matches, config),
-        use_basic_mode,
-        default_time_value,
-        time_interval: get_time_interval(matches, config)
-            .context("Update 'time_delta' in your config file.")?,
-        hide_time: get_hide_time(matches, config),
-        autohide_time,
-        use_old_network_legend: get_use_old_network_legend(matches, config),
-        table_gap: if get_hide_table_gap(matches, config) {
-            0
-        } else {
-            1
-        },
-        disable_click: get_disable_click(matches, config),
-        // no_write: get_no_write(matches, config),
-        no_write: false,
-        show_table_scroll_position: get_show_table_scroll_position(matches, config),
-        is_advanced_kill,
-        network_scale_type,
-        network_unit_type,
-        network_use_binary_prefix,
-    };
-
+    let use_mem = used_widget_set.get(&Mem).is_some() || used_widget_set.get(&BasicMem).is_some();
     let used_widgets = UsedWidgets {
         use_cpu: used_widget_set.get(&Cpu).is_some() || used_widget_set.get(&BasicCpu).is_some(),
-        use_mem: used_widget_set.get(&Mem).is_some() || used_widget_set.get(&BasicMem).is_some(),
+        use_mem,
+        use_gpu: use_mem && get_enable_gpu_memory(matches, config),
         use_net: used_widget_set.get(&Net).is_some() || used_widget_set.get(&BasicNet).is_some(),
         use_proc: used_widget_set.get(&Proc).is_some(),
         use_disk: used_widget_set.get(&Disk).is_some(),
@@ -457,47 +403,7 @@ pub fn build_app(
     let net_filter =
         get_ignore_list(&config.net_filter).context("Update 'net_filter' in your config file")?;
 
-    // One more thing - we have to update the search settings of our proc_state_map, and create the hashmaps if needed!
-    // Note that if you change your layout, this might not actually match properly... not sure if/where we should deal with that...
-    if let Some(flags) = &mut config.flags {
-        if flags.case_sensitive.is_none() && !matches.is_present("case_sensitive") {
-            if let Some(search_case_enabled_widgets) = &flags.search_case_enabled_widgets {
-                let mapping = HashMap::new();
-                for widget in search_case_enabled_widgets {
-                    if let Some(proc_widget) = proc_state_map.get_mut(&widget.id) {
-                        proc_widget.process_search_state.is_ignoring_case = !widget.enabled;
-                    }
-                }
-                flags.search_case_enabled_widgets_map = Some(mapping);
-            }
-        }
-
-        if flags.whole_word.is_none() && !matches.is_present("whole_word") {
-            if let Some(search_whole_word_enabled_widgets) =
-                &flags.search_whole_word_enabled_widgets
-            {
-                let mapping = HashMap::new();
-                for widget in search_whole_word_enabled_widgets {
-                    if let Some(proc_widget) = proc_state_map.get_mut(&widget.id) {
-                        proc_widget.process_search_state.is_searching_whole_word = widget.enabled;
-                    }
-                }
-                flags.search_whole_word_enabled_widgets_map = Some(mapping);
-            }
-        }
-
-        if flags.regex.is_none() && !matches.is_present("regex") {
-            if let Some(search_regex_enabled_widgets) = &flags.search_regex_enabled_widgets {
-                let mapping = HashMap::new();
-                for widget in search_regex_enabled_widgets {
-                    if let Some(proc_widget) = proc_state_map.get_mut(&widget.id) {
-                        proc_widget.process_search_state.is_searching_with_regex = widget.enabled;
-                    }
-                }
-                flags.search_regex_enabled_widgets_map = Some(mapping);
-            }
-        }
-    }
+    let expanded_upon_startup = get_expanded_on_startup(matches, config);
 
     Ok(App::builder()
         .app_config_fields(app_config_fields)
@@ -512,19 +418,18 @@ pub fn build_app(
         .current_widget(widget_map.get(&initial_widget_id).unwrap().clone()) // TODO: [UNWRAP] - many of the unwraps are fine (like this one) but do a once-over and/or switch to expect?
         .widget_map(widget_map)
         .used_widgets(used_widgets)
+        .is_expanded(expanded_upon_startup && !use_basic_mode)
         .filters(DataFilters {
             disk_filter,
             mount_filter,
             temp_filter,
             net_filter,
         })
-        .config(config.clone())
-        .config_path(config_path)
         .build())
 }
 
 pub fn get_widget_layout(
-    matches: &clap::ArgMatches<'static>, config: &Config,
+    matches: &ArgMatches, config: &Config,
 ) -> error::Result<(BottomLayout, u64, Option<BottomWidgetType>)> {
     let left_legend = get_use_left_legend(matches, config);
     let (default_widget_type, mut default_widget_count) =
@@ -588,36 +493,34 @@ pub fn get_widget_layout(
     Ok((bottom_layout, default_widget_id, default_widget_type))
 }
 
-fn get_update_rate_in_milliseconds(
-    matches: &clap::ArgMatches<'static>, config: &Config,
-) -> error::Result<u64> {
+fn get_update_rate_in_milliseconds(matches: &ArgMatches, config: &Config) -> error::Result<u64> {
     let update_rate_in_milliseconds = if let Some(update_rate) = matches.value_of("rate") {
-        update_rate.parse::<u128>()?
+        update_rate.parse::<u64>().map_err(|_| {
+            BottomError::ConfigError(
+                "could not parse as a valid 64-bit unsigned integer".to_string(),
+            )
+        })?
     } else if let Some(flags) = &config.flags {
         if let Some(rate) = flags.rate {
-            rate as u128
+            rate
         } else {
-            DEFAULT_REFRESH_RATE_IN_MILLISECONDS as u128
+            DEFAULT_REFRESH_RATE_IN_MILLISECONDS
         }
     } else {
-        DEFAULT_REFRESH_RATE_IN_MILLISECONDS as u128
+        DEFAULT_REFRESH_RATE_IN_MILLISECONDS
     };
 
     if update_rate_in_milliseconds < 250 {
         return Err(BottomError::ConfigError(
             "set your update rate to be at least 250 milliseconds.".to_string(),
         ));
-    } else if update_rate_in_milliseconds as u128 > std::u64::MAX as u128 {
-        return Err(BottomError::ConfigError(
-            "set your update rate to be at most unsigned INT_MAX.".to_string(),
-        ));
     }
 
-    Ok(update_rate_in_milliseconds as u64)
+    Ok(update_rate_in_milliseconds)
 }
 
 fn get_temperature(
-    matches: &clap::ArgMatches<'static>, config: &Config,
+    matches: &ArgMatches, config: &Config,
 ) -> error::Result<data_harvester::temperature::TemperatureType> {
     if matches.is_present("fahrenheit") {
         return Ok(data_harvester::temperature::TemperatureType::Fahrenheit);
@@ -643,7 +546,7 @@ fn get_temperature(
 }
 
 /// Yes, this function gets whether to show average CPU (true) or not (false)
-fn get_show_average_cpu(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
+fn get_show_average_cpu(matches: &ArgMatches, config: &Config) -> bool {
     if matches.is_present("hide_avg_cpu") {
         return false;
     } else if let Some(flags) = &config.flags {
@@ -655,7 +558,7 @@ fn get_show_average_cpu(matches: &clap::ArgMatches<'static>, config: &Config) ->
     true
 }
 
-fn get_use_dot(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
+fn get_use_dot(matches: &ArgMatches, config: &Config) -> bool {
     if matches.is_present("dot_marker") {
         return true;
     } else if let Some(flags) = &config.flags {
@@ -666,7 +569,7 @@ fn get_use_dot(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
     false
 }
 
-fn get_use_left_legend(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
+fn get_use_left_legend(matches: &ArgMatches, config: &Config) -> bool {
     if matches.is_present("left_legend") {
         return true;
     } else if let Some(flags) = &config.flags {
@@ -678,7 +581,7 @@ fn get_use_left_legend(matches: &clap::ArgMatches<'static>, config: &Config) ->
     false
 }
 
-fn get_use_current_cpu_total(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
+fn get_use_current_cpu_total(matches: &ArgMatches, config: &Config) -> bool {
     if matches.is_present("current_usage") {
         return true;
     } else if let Some(flags) = &config.flags {
@@ -690,7 +593,19 @@ fn get_use_current_cpu_total(matches: &clap::ArgMatches<'static>, config: &Confi
     false
 }
 
-fn get_use_basic_mode(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
+fn get_unnormalized_cpu(matches: &ArgMatches, config: &Config) -> bool {
+    if matches.is_present("unnormalized_cpu") {
+        return true;
+    } else if let Some(flags) = &config.flags {
+        if let Some(unnormalized_cpu) = flags.unnormalized_cpu {
+            return unnormalized_cpu;
+        }
+    }
+
+    false
+}
+
+fn get_use_basic_mode(matches: &ArgMatches, config: &Config) -> bool {
     if matches.is_present("basic") {
         return true;
     } else if let Some(flags) = &config.flags {
@@ -702,63 +617,74 @@ fn get_use_basic_mode(matches: &clap::ArgMatches<'static>, config: &Config) -> b
     false
 }
 
+/// FIXME: Let this accept human times.
 fn get_default_time_value(
-    matches: &clap::ArgMatches<'static>, config: &Config,
+    matches: &ArgMatches, config: &Config, retention_ms: u64,
 ) -> error::Result<u64> {
     let default_time = if let Some(default_time_value) = matches.value_of("default_time_value") {
-        default_time_value.parse::<u128>()?
+        default_time_value.parse::<u64>().map_err(|_| {
+            BottomError::ConfigError(
+                "could not parse as a valid 64-bit unsigned integer".to_string(),
+            )
+        })?
     } else if let Some(flags) = &config.flags {
         if let Some(default_time_value) = flags.default_time_value {
-            default_time_value as u128
+            default_time_value
         } else {
-            DEFAULT_TIME_MILLISECONDS as u128
+            DEFAULT_TIME_MILLISECONDS
         }
     } else {
-        DEFAULT_TIME_MILLISECONDS as u128
+        DEFAULT_TIME_MILLISECONDS
     };
 
     if default_time < 30000 {
         return Err(BottomError::ConfigError(
             "set your default value to be at least 30000 milliseconds.".to_string(),
         ));
-    } else if default_time as u128 > STALE_MAX_MILLISECONDS as u128 {
+    } else if default_time > retention_ms {
         return Err(BottomError::ConfigError(format!(
             "set your default value to be at most {} milliseconds.",
-            STALE_MAX_MILLISECONDS
+            retention_ms
         )));
     }
 
-    Ok(default_time as u64)
+    Ok(default_time)
 }
 
-fn get_time_interval(matches: &clap::ArgMatches<'static>, config: &Config) -> error::Result<u64> {
+fn get_time_interval(
+    matches: &ArgMatches, config: &Config, retention_ms: u64,
+) -> error::Result<u64> {
     let time_interval = if let Some(time_interval) = matches.value_of("time_delta") {
-        time_interval.parse::<u128>()?
+        time_interval.parse::<u64>().map_err(|_| {
+            BottomError::ConfigError(
+                "could not parse as a valid 64-bit unsigned integer".to_string(),
+            )
+        })?
     } else if let Some(flags) = &config.flags {
         if let Some(time_interval) = flags.time_delta {
-            time_interval as u128
+            time_interval
         } else {
-            TIME_CHANGE_MILLISECONDS as u128
+            TIME_CHANGE_MILLISECONDS
         }
     } else {
-        TIME_CHANGE_MILLISECONDS as u128
+        TIME_CHANGE_MILLISECONDS
     };
 
     if time_interval < 1000 {
         return Err(BottomError::ConfigError(
             "set your time delta to be at least 1000 milliseconds.".to_string(),
         ));
-    } else if time_interval > STALE_MAX_MILLISECONDS as u128 {
+    } else if time_interval > retention_ms {
         return Err(BottomError::ConfigError(format!(
             "set your time delta to be at most {} milliseconds.",
-            STALE_MAX_MILLISECONDS
+            retention_ms
         )));
     }
 
-    Ok(time_interval as u64)
+    Ok(time_interval)
 }
 
-pub fn get_app_grouping(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
+pub fn get_app_grouping(matches: &ArgMatches, config: &Config) -> bool {
     if matches.is_present("group") {
         return true;
     } else if let Some(flags) = &config.flags {
@@ -769,7 +695,7 @@ pub fn get_app_grouping(matches: &clap::ArgMatches<'static>, config: &Config) ->
     false
 }
 
-pub fn get_app_case_sensitive(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
+pub fn get_app_case_sensitive(matches: &ArgMatches, config: &Config) -> bool {
     if matches.is_present("case_sensitive") {
         return true;
     } else if let Some(flags) = &config.flags {
@@ -780,7 +706,7 @@ pub fn get_app_case_sensitive(matches: &clap::ArgMatches<'static>, config: &Conf
     false
 }
 
-pub fn get_app_match_whole_word(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
+pub fn get_app_match_whole_word(matches: &ArgMatches, config: &Config) -> bool {
     if matches.is_present("whole_word") {
         return true;
     } else if let Some(flags) = &config.flags {
@@ -791,7 +717,7 @@ pub fn get_app_match_whole_word(matches: &clap::ArgMatches<'static>, config: &Co
     false
 }
 
-pub fn get_app_use_regex(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
+pub fn get_app_use_regex(matches: &ArgMatches, config: &Config) -> bool {
     if matches.is_present("regex") {
         return true;
     } else if let Some(flags) = &config.flags {
@@ -802,7 +728,7 @@ pub fn get_app_use_regex(matches: &clap::ArgMatches<'static>, config: &Config) -
     false
 }
 
-fn get_hide_time(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
+fn get_hide_time(matches: &ArgMatches, config: &Config) -> bool {
     if matches.is_present("hide_time") {
         return true;
     } else if let Some(flags) = &config.flags {
@@ -813,7 +739,7 @@ fn get_hide_time(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
     false
 }
 
-fn get_autohide_time(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
+fn get_autohide_time(matches: &ArgMatches, config: &Config) -> bool {
     if matches.is_present("autohide_time") {
         return true;
     } else if let Some(flags) = &config.flags {
@@ -825,8 +751,17 @@ fn get_autohide_time(matches: &clap::ArgMatches<'static>, config: &Config) -> bo
     false
 }
 
+fn get_expanded_on_startup(matches: &ArgMatches, config: &Config) -> bool {
+    matches.is_present("expanded_on_startup")
+        || config
+            .flags
+            .as_ref()
+            .and_then(|x| x.expanded_on_startup)
+            .unwrap_or(false)
+}
+
 fn get_default_widget_and_count(
-    matches: &clap::ArgMatches<'static>, config: &Config,
+    matches: &ArgMatches, config: &Config,
 ) -> error::Result<(Option<BottomWidgetType>, u64)> {
     let widget_type = if let Some(widget_type) = matches.value_of("default_widget_type") {
         let parsed_widget = widget_type.parse::<BottomWidgetType>()?;
@@ -855,20 +790,17 @@ fn get_default_widget_and_count(
     } else if let Some(flags) = &config.flags {
         flags
             .default_widget_count
-            .map(|widget_count| widget_count as u128)
+            .map(|widget_count| widget_count.into())
     } else {
         None
     };
 
     match (widget_type, widget_count) {
         (Some(widget_type), Some(widget_count)) => {
-            if widget_count > std::u64::MAX as u128 {
-                Err(BottomError::ConfigError(
-                    "set your widget count to be at most unsigned INT_MAX.".to_string(),
-                ))
-            } else {
-                Ok((Some(widget_type), widget_count as u64))
-            }
+            let widget_count = widget_count.try_into().map_err(|_| BottomError::ConfigError(
+                "set your widget count to be at most unsigned INT_MAX.".to_string()
+            ))?;
+            Ok((Some(widget_type), widget_count))
         }
         (Some(widget_type), None) => Ok((Some(widget_type), 1)),
         (None, Some(_widget_count)) =>  Err(BottomError::ConfigError(
@@ -878,7 +810,7 @@ fn get_default_widget_and_count(
     }
 }
 
-fn get_disable_click(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
+fn get_disable_click(matches: &ArgMatches, config: &Config) -> bool {
     if matches.is_present("disable_click") {
         return true;
     } else if let Some(flags) = &config.flags {
@@ -889,7 +821,7 @@ fn get_disable_click(matches: &clap::ArgMatches<'static>, config: &Config) -> bo
     false
 }
 
-fn get_use_old_network_legend(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
+fn get_use_old_network_legend(matches: &ArgMatches, config: &Config) -> bool {
     if matches.is_present("use_old_network_legend") {
         return true;
     } else if let Some(flags) = &config.flags {
@@ -900,7 +832,7 @@ fn get_use_old_network_legend(matches: &clap::ArgMatches<'static>, config: &Conf
     false
 }
 
-fn get_hide_table_gap(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
+fn get_hide_table_gap(matches: &ArgMatches, config: &Config) -> bool {
     if matches.is_present("hide_table_gap") {
         return true;
     } else if let Some(flags) = &config.flags {
@@ -911,19 +843,34 @@ fn get_hide_table_gap(matches: &clap::ArgMatches<'static>, config: &Config) -> b
     false
 }
 
-fn get_use_battery(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
-    if matches.is_present("battery") {
-        return true;
-    } else if let Some(flags) = &config.flags {
-        if let Some(battery) = flags.battery {
-            return battery;
+fn get_use_battery(matches: &ArgMatches, config: &Config) -> bool {
+    if cfg!(feature = "battery") {
+        if matches.is_present("battery") {
+            return true;
+        } else if let Some(flags) = &config.flags {
+            if let Some(battery) = flags.battery {
+                return battery;
+            }
+        }
+    }
+    false
+}
+
+fn get_enable_gpu_memory(matches: &ArgMatches, config: &Config) -> bool {
+    if cfg!(feature = "gpu") {
+        if matches.is_present("enable_gpu_memory") {
+            return true;
+        } else if let Some(flags) = &config.flags {
+            if let Some(enable_gpu_memory) = flags.enable_gpu_memory {
+                return enable_gpu_memory;
+            }
         }
     }
     false
 }
 
 #[allow(dead_code)]
-fn get_no_write(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
+fn get_no_write(matches: &ArgMatches, config: &Config) -> bool {
     if matches.is_present("no_write") {
         return true;
     } else if let Some(flags) = &config.flags {
@@ -971,9 +918,7 @@ fn get_ignore_list(ignore_list: &Option<IgnoreList>) -> error::Result<Option<Fil
     }
 }
 
-pub fn get_color_scheme(
-    matches: &clap::ArgMatches<'static>, config: &Config,
-) -> error::Result<ColourScheme> {
+pub fn get_color_scheme(matches: &ArgMatches, config: &Config) -> error::Result<ColourScheme> {
     if let Some(color) = matches.value_of("color") {
         // Highest priority is always command line flags...
         return ColourScheme::from_str(color);
@@ -998,7 +943,7 @@ pub fn get_color_scheme(
     Ok(ColourScheme::Default)
 }
 
-fn get_mem_as_value(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
+fn get_mem_as_value(matches: &ArgMatches, config: &Config) -> bool {
     if matches.is_present("mem_as_value") {
         return true;
     } else if let Some(flags) = &config.flags {
@@ -1009,7 +954,7 @@ fn get_mem_as_value(matches: &clap::ArgMatches<'static>, config: &Config) -> boo
     false
 }
 
-fn get_is_default_tree(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
+fn get_is_default_tree(matches: &ArgMatches, config: &Config) -> bool {
     if matches.is_present("tree") {
         return true;
     } else if let Some(flags) = &config.flags {
@@ -1020,7 +965,7 @@ fn get_is_default_tree(matches: &clap::ArgMatches<'static>, config: &Config) ->
     false
 }
 
-fn get_show_table_scroll_position(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
+fn get_show_table_scroll_position(matches: &ArgMatches, config: &Config) -> bool {
     if matches.is_present("show_table_scroll_position") {
         return true;
     } else if let Some(flags) = &config.flags {
@@ -1031,7 +976,7 @@ fn get_show_table_scroll_position(matches: &clap::ArgMatches<'static>, config: &
     false
 }
 
-fn get_is_default_process_command(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
+fn get_is_default_process_command(matches: &ArgMatches, config: &Config) -> bool {
     if matches.is_present("process_command") {
         return true;
     } else if let Some(flags) = &config.flags {
@@ -1042,7 +987,7 @@ fn get_is_default_process_command(matches: &clap::ArgMatches<'static>, config: &
     false
 }
 
-fn get_is_advanced_kill_disabled(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
+fn get_is_advanced_kill_disabled(matches: &ArgMatches, config: &Config) -> bool {
     if matches.is_present("disable_advanced_kill") {
         return true;
     } else if let Some(flags) = &config.flags {
@@ -1053,7 +998,7 @@ fn get_is_advanced_kill_disabled(matches: &clap::ArgMatches<'static>, config: &C
     false
 }
 
-fn get_network_unit_type(matches: &clap::ArgMatches<'static>, config: &Config) -> DataUnit {
+fn get_network_unit_type(matches: &ArgMatches, config: &Config) -> DataUnit {
     if matches.is_present("network_use_bytes") {
         return DataUnit::Byte;
     } else if let Some(flags) = &config.flags {
@@ -1067,7 +1012,7 @@ fn get_network_unit_type(matches: &clap::ArgMatches<'static>, config: &Config) -
     DataUnit::Bit
 }
 
-fn get_network_scale_type(matches: &clap::ArgMatches<'static>, config: &Config) -> AxisScaling {
+fn get_network_scale_type(matches: &ArgMatches, config: &Config) -> AxisScaling {
     if matches.is_present("network_use_log") {
         return AxisScaling::Log;
     } else if let Some(flags) = &config.flags {
@@ -1081,7 +1026,7 @@ fn get_network_scale_type(matches: &clap::ArgMatches<'static>, config: &Config)
     AxisScaling::Linear
 }
 
-fn get_network_use_binary_prefix(matches: &clap::ArgMatches<'static>, config: &Config) -> bool {
+fn get_network_use_binary_prefix(matches: &ArgMatches, config: &Config) -> bool {
     if matches.is_present("network_use_binary_prefix") {
         return true;
     } else if let Some(flags) = &config.flags {
@@ -1091,3 +1036,21 @@ fn get_network_use_binary_prefix(matches: &clap::ArgMatches<'static>, config: &C
     }
     false
 }
+
+fn get_retention_ms(matches: &ArgMatches, config: &Config) -> error::Result<u64> {
+    const DEFAULT_RETENTION_MS: u64 = 600 * 1000; // Keep 10 minutes of data.
+
+    if let Some(retention) = matches.value_of("retention") {
+        humantime::parse_duration(retention)
+            .map(|dur| dur.as_millis() as u64)
+            .map_err(|err| BottomError::ConfigError(format!("invalid retention duration: {err:?}")))
+    } else if let Some(flags) = &config.flags {
+        if let Some(retention) = flags.retention {
+            Ok(retention.as_millis() as u64)
+        } else {
+            Ok(DEFAULT_RETENTION_MS)
+        }
+    } else {
+        Ok(DEFAULT_RETENTION_MS)
+    }
+}
