@@ -1,5 +1,5 @@
-use beef::Cow;
-use std::result;
+use std::{borrow::Cow, result};
+
 use thiserror::Error;
 
 #[cfg(target_os = "linux")]
@@ -9,7 +9,7 @@ use procfs::ProcError;
 pub type Result<T> = result::Result<T, BottomError>;
 
 /// An error that can occur while Bottom runs.
-#[derive(Debug, Error)]
+#[derive(Debug, Error, PartialEq, Eq)]
 pub enum BottomError {
     /// An error when there is an IO exception.
     #[error("IO exception, {0}")]
@@ -50,18 +50,13 @@ impl From<std::io::Error> for BottomError {
     }
 }
 
+#[cfg(not(target_os = "freebsd"))]
 impl From<heim::Error> for BottomError {
     fn from(err: heim::Error) -> Self {
         BottomError::InvalidHeim(err.to_string())
     }
 }
 
-impl From<crossterm::ErrorKind> for BottomError {
-    fn from(err: crossterm::ErrorKind) -> Self {
-        BottomError::CrosstermError(err.to_string())
-    }
-}
-
 impl From<std::num::ParseIntError> for BottomError {
     fn from(err: std::num::ParseIntError) -> Self {
         BottomError::ConfigError(err.to_string())
@@ -105,13 +100,7 @@ impl From<regex::Error> for BottomError {
         let err_str = err.to_string();
         let error = err_str.split('\n').map(|s| s.trim()).collect::<Vec<_>>();
 
-        BottomError::QueryError(
-            format!(
-                "Regex error: {}",
-                error.last().unwrap_or(&"".to_string().as_str())
-            )
-            .into(),
-        )
+        BottomError::QueryError(format!("Regex error: {}", error.last().unwrap_or(&"")).into())
     }
 }
 
